%
%   This function constructs a block-hankel matrix.
%
%   Input: c(:,:,k) = ck, k = 1..nc
%          r(:,:,k) = rk, k = 1..nr
%                  [ ri11 ... ri1p ]
%             ri = [  .         .  ]
%                  [ riq1 ... riqp ]
%
%               [ c0 c1 ...  cn ] 
%   Output: t = [ c1 c2 ...  r2 ]
%               [ :  :       :  ]
%               [ cn    ...  rn ]
%
function t = BlkHankel(c,r)

if nargin < 2
    r = a*0;
    r(:,:,1) = c(:,:,end)
end

[ qc pc nc ] = size(c);
[ qr pr nr ] = size(r);

if qr ~= qc | pr ~= pc
    error('In BlkHankel: different dimiensions')
    return
end

if norm(c(:,:,end)-r(:,:,1)) > 1e-12
    error('In BlkHankel: diagonal conflict')
    return
end

x = [];
for k = 1:nc
    x = [ x; c(:,:,k) ];
end
for k = 2:nr
    x = [ x; r(:,:,k) ];
end

cidx = (1:qc*nc)';
ridx = [];
for k = 0:(nr-1)
    ridx = [ ridx k*qr:size(x,1):(prod(size(x))-1) ];
end

t = cidx(:,ones(pr*nr,1)) + ridx(ones(qc*nc,1),:);
t(:) = x(t);

