%
%	This script saves the Hinf design and the corresponding model 
%    reduced version in "AIRCHinfdesign.mat". For more details see
%    ExAIRCmain.m.
%
clear all, format compact

%
%   Load plant data
%
load AIRCmodel

%
%   Construction shaping filter W1 & W2 for Hinf design in anology with example
%     on p. 307.
%

w1 = tf([1 12 36],[1 .60006 .000036]);
W1 = w1*eye(3);
w2 = tf(2000*[1 60 500],[1 2000 1000000]);
W2 = w2*eye(3);

%
%   We modify the system to get rid of some poles closed to the imaginary axis. 
%     However, the modified system will be very close to the original in at least
%     .001 < w < 100, which will be enough. This will enable inner-outer 
%     factorization.
%

Atilde =  A;
Atilde(1,1) = -.001;    % moves the pole from the imaginay axis
Dtilde = 1e-5*eye(3);   % fulfill rank condition in Glover-Doyle  
systilde = ss(Atilde,B,C,Dtilde);

G2sys = [ W1 -series(systilde,W1); ...
        zeros(3,3) series(systilde,W2); ...
        eye(3) -systilde];
[a b c d] = ssdata(minreal(G2sys));
G2 = ltisys(a,b,c,d);
sinfo(G2);

%
%   Controller design using standard Hinf synthesis
%
disp('Using Standard Hinf Control Synthesis')
[gopt,KHinf,x1,x2,y1,y2,Preg] = hinfric(G2,[3 3],5);
clsysHinf = slft(G,KHinf);
THinf = smult(clsysHinf,smult(KHinf,P));

%
%   Save the data
%
clear A     B     C     D     G     Gsys  P     a     b     c     d     sys
save AIRCHinfdesign