%
%   This program runs the a filter example
%
clear all, format compact, figure(1), clf, hold on

%
%   Sizes
%
testruns = 1;
n = 4;
samples = 500;
flag = 2;

%
%   Given filter
%
if flag == 1
    bin = [ 1 -1.051   0.0718 0.05164 0.5322 -0.5735 ];
    ain = [ 1 -0.8713 -1.539  1.371   0.6451 -0.5827 ];
    sys = tf(bin,ain,1);
else
    sys(1,1) = tf([ 1 0.5451 0.8406 ],[ 1 -0.8936 0.9702 ],1);
    sys(2,1) = tf([ 0 0.8334 0.02877],[ 1 -0.8936 0.9702 ],1);
    sys(1,2) = tf([ 0 0.8854 0.1189 ],[ 1  0.7600 0.9860 ],1);
    sys(2,2) = tf([ 1 0.5053 0.7187 ],[ 1  0.7600 0.9860 ],1);
end

randn('state',0);
tvec = 0:samples*100-1;
noise = randn(flag,samples*100);
[yvec,tvec] = lsim(sys,noise,tvec);
x = yvec((samples*100-samples):end,:);

C = xcorr(x);
R = rceps(x);
R = [R(1,:); 2*R(2:end,:)];
C(1:length(x)-1,:) = [];
C = reshape(C',flag,flag,samples+1);

%
%   Get interdata
%
W{1} = {C(:,:,1)/2};
for k = 2:n+1
    W{1} = [ W{1} {C(:,:,k)} ];
end

%
% Plot
%
figure(1), clf, hold on
w = 0:pi/100:pi;
N = 2*length(w)-1;
[sv w] = sigma(sys,w);
plottrue = plot(w,20*log10(abs(sv)),'k-');

% y = fft(x,2*length(w)-1);
% for k = 1:2
%     Pyy = pwelch(x(:,k),hamming(N),0,N);
%     plot(w,20*log10(Pyy),'k:')
% end

xlim([0,pi]);
xlabel('Frequency (rad/s)')
ylabel('Gain (dB)')

%
%   get roots of cepstrum
%
speczerovec = zeros(n,1);
for k = 1:flag
    tmp = roots(R(1:n+1,k));
    [ xx idx ] = sort(real(tmp));
    speczerovec = speczerovec + tmp(idx);
end
speczerovec = speczerovec' /flag;

sysNPDC = {};
for mult = [0 1]

%    speczeros = mult * speczerovec; 
    speczeros = mult * [ -.7+.6i .2+.8i ]; 

    interdata = GetInterData(0,W,speczeros,1);
    interpolant = NPDC(interdata);
    [z p k] = zpkdata(interpolant.sysV);
    sysNPDC{end+1} = inv(k)*interpolant.sysV;
end

[sv w] = sigma(sysNPDC{1},w);
plotcentral = plot(w,20*log10(abs(sv)),'k:');
[sv w] = sigma(sysNPDC{2},w);
plotARMA = plot(w,20*log10(abs(sv)),'k--');
legend([plottrue(1) plotcentral(1) plotARMA(1)],'True filter','AR-model','ARMA-model',3)

%
%   Printing
%
printflag = 1;
if printflag
    papersize = [ 10 7.5 ];
    width = 7;
    height = 5;
    left = (papersize(1)- width) / 2;
    bottom = (papersize(2)- height) / 2;
    myfiguresize = [left, bottom, width, height];
    myfontsize = 12;
    myfontweight = 'bold';

    figure(1)
    set(gcf,'Units','centimeters')
    set(gcf, 'PaperPosition', myfiguresize);
    ChangeTextStyle(gcf,myfontsize,myfontweight);
    print spectrum -deps
        
end