%
%   This function calculates the gradient of g_rho (or g~_rho)
%    at the point (A, mu)
%
%   Input: A(:,:,k) -  Ak, k = 0..n
%          muvec    - [ mu0 mu1 ... mun ]'
%          tau      - [ tau0 tau1 ... taun ]
%          K        - modified Pick matrix
%
%       where A(z) = A0 + A1 z + ... + An z^n
%             mu(z) = mu0 + mu1 (z + z^-1) + ... + mun (z^n + z^-n)
%             tau(z) = tau0 + tau1 z + ... + taun z^n
%
%   Output: grad - gradient
%
function grad = GetGradient(A,muvec,tau,K)

ntau = length(tau);
n = size(A,3) - 1;
m = size(A,2);

Atau = MPolyMult(reshape(tau,1,1,ntau),A);
chat = GetARCovariances(Atau,n+ntau-1);
chat = ARtimesMA(muvec,chat);

Abold = reshape(A,m,m*(n+1));
C = BlkToeplitz(BlkTrans(chat(:,:,1:end)),chat(:,:,1:end));
grad = (2 * Abold * (K - C))';
grad = grad(:);
