%
%   This function checks whether the interpolation conditions
%    are satisfied or not.
%
%       F + F* = Y^-*Y^-1 
%              = (R*R/rho*rho)^-1 
%              = p (R*R/tau*tau)^-1
%
%   Input: interpolant - an interpolant data structure
%          interdata   - an interpolation data structure
%
%   Output: interror - the current interpolation error
%
function interror = GetIntError(interpolant,interdata)

errorvec = [];
for k = 1:interdata.ntilde
    
    z = interdata.z(k);
    
    errorvec = [ errorvec norm( interpolant.D + interpolant.C*z / ...
        (eye(size(interpolant.A,1)) - interpolant.A*z) * interpolant.B ...
        - interdata.W{k}{1}) ];

    for l = 1:interdata.mw(k)-1
        errorvec = [ errorvec norm( factorial(l)*(interpolant.C * interpolant.A^(l-1) ...
            / ((eye(size(interpolant.A,1)) - interpolant.A*z)^(l+1)) * interpolant.B ...
            - interdata.W{k}{l+1})) ];
    end
end

interror = norm(errorvec);