%
%   This function inverts a 3-by-3 rational transfer matrix given as a system.
%    The inverse is returned as a system on tranfer function form and is not 
%    necessarily proper (so a statespace represenation might not exist).
%
%   Indata: sys - an LTI system of the Control System Toolbox form with three
%                  inputs and three outputs.
%
%   Outdata: invsys - an LTI sysstem of the transfer function form of the Control
%                      System Toolbox.
%
function sysinv = Inv3by3(sys)

[num, den] = tfdata(sys);

sizes = size(num);

maxlength = 0;
for k = 1:sizes(1)
    for l = 1:sizes(2)
        maxlength = max(maxlength,length(den{k,l}));
    end
end

for k = 1:sizes(1)
    for l = 1:sizes(2)
        if length(den{k,l}) < maxlength
            add = zeros(1,maxlength-length(den{k,l}));
            den{k,l} = [den{k,l} add];
            num{k,l} = [num{k,l} add];
        end
    end
end

minfirstnonzero = 1e10;
for k = 1:3
    index1 = find([1 2 3] ~= k);
    for l = 1:3
        index2 = find([1 2 3] ~= l);
        newnum{l,k} = (-1)^(k+l) * (conv(num{index1(1),index2(1)},num{index1(2),index2(2)}) ...
            - conv(num{index1(1),index2(2)},num{index1(2),index2(1)}));
        if sum(newnum{l,k} ~= 0*newnum{l,k}) ~= 0
            minfirstnonzero = min(minfirstnonzero,min(find(newnum{l,k})));
        end
    end
end

for k = 1:3
    for l = 1:3
        newnum{l,k} = newnum{l,k}(minfirstnonzero:end);
    end
end

det = conv(num{1,1},newnum{1,1}) + conv(num{1,2},newnum{2,1}) + conv(num{1,3},newnum{3,1});

xi = minreal(tf(den{1,1},det));
[numxi, denxi] = tfdata(xi);

numxi = numxi{1};
while numxi(1) == 0
    numxi = numxi(2:end);
end
if length(numxi) ~= 1
    numxi = tf(numxi,1);
end

denxi = denxi{1};
if length(denxi) < length(newnum{1,1})
    denxi = [zeros(1,length(newnum{1,1})-length(denxi)) denxi];
elseif length(denxi) > length(newnum{1,1})
    add = zeros(1,length(denxi) - length(newnum{1,1}));
    for k = 1:3
        for l = 1:3
            newnum{l,k} = [ add newnum{l,k} ]; 
        end
    end
end

sysinv = numxi*tf(newnum,denxi);
